#!/usr/bin/perl -T
#------------------------------------------------------------
# test serveur CGI (TvO)
#------------------------------------------------------------

#------------------------------------------------------------
# packages
#------------------------------------------------------------
use strict;
use warnings;
use CGI qw(param Dump delete standards) ;
use CGI::Carp ('fatalsToBrowser');
use Cwd;
use File::Copy;
use File::Listing qw(parse_dir);
use File::Temp qw(tempfile);
use Template;
use XML::Twig;
#use Date::Calc qw(Decode_Date_EU Today Date_to_Text);

#------------------------------------------------------------
# variables, constantes,...
#------------------------------------------------------------
my $NomCgi = "index.cgi";
my $repository = "/usr/local/www/data/rep_cgi/";
$repository =~ s/^(.*)$/$1/;
my $login_chef;
my $chef = "dupont";
my %permanents = (
		  aldanond => "Michel Aldanondo",
		  benaben  => "Frdrick Benaben",
		  dupont   => "Lionel Dupont",
		  fontanil => "Franck Fontanili",
		  ifournie => "Isabelle Fournier",
		  gaborit  => "Paul Gaborit",
		  gourc    => "Didier Gourc",
		  lamothe  => "Jacques Lamothe",
		  lauras   => "Matthieu Lauras",
		  jmarey   => "Jacqueline Marey",
		  matha    => "Jacqueline Matha",
		  meyermey => "Marc-Andr Meyer",
		  pingaud  => "Herv Pingaud",
		  vareille => "Elise Vareilles",
		 );
my %login_valides = (
		     deguil   => "Romain Deguil",
		     vrajsiri => "Vatcharaphun Rajsiri",
		     vehumezh => "Verane Humez",
		     mahmoudi => "Jaouher Mahmoudi",
		     mirdamad => "Samieh Mirdamadi",
		     sienou   => "Amadou Sienou",
		     touzitou => "Jihed Touzi",
		     vanouden => "Thomas van Oudenhove",
		    );
my $ListeJour = [1..31];
my $ListeMois = {
		 "01" => "janvier",
		 "02" => "fvrier",
		 "03" => "mars",
		 "04" => "avril",
		 "05" => "mai",
		 "06" => "juin",
		 "07" => "juillet",
		 "08" => "aot",
		 "09" => "septembre",
		 "10" => "octobre",
		 "11" => "novembre",
		 "12" => "dcembre",
		};
my $ListeAnnee = [2005..2010];
my %action =
  (
   menu   	    => \&choix_action,
   retour 	    => \&choix_action,
   login  	    => \&login,
   saisie 	    => \&saisie,
   saisie_chef      => \&saisie_chef,
   modification     => \&modif,
   consultation     => \&consult,
   choix_rapport    => \&select_rap,
   verification     => \&verif_saisie,
   generation       => \&creer_rapport,
   generation_chef  => \&creer_rapport_chef,
   afficher_rapport => \&list_rap_to_process,
  );

# # REMINDER...
# $donnees = {
# 	    query 	=> CGI-new(),
# 	    login 	=> le login du connect,
# 	    rapport 	=> le rapport qu'on veut voir,
# 	    warn_saisie => avertissement suite  la saisie,
# 	    warn_login 	=> avertissement suite au login,
# 	    login_chef 	=> 1 si $login = $chef,
# 	    action 	=> action  effectuer,
# 	    periode 	=> (annee, mois, jour),
# 	    datereunion => ((annee, mois), (annee, mois))
#  	   };

#------------------------------------------------------------
# execution du programme principal ... et arrt
#------------------------------------------------------------
$ENV{'PATH'} = '/bin:/usr/bin';
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};

warn "*****************************\n";
main();
exit 0;

#------------------------------------------------------------
# le main.
#------------------------------------------------------------
sub main {
  my $donnees = {
		 query => CGI->new(),
		};
  $donnees->{dump_debug} = $donnees->{query}->Dump();
  $donnees->{login} = $donnees->{query}->param('login');
#  $donnees->{date_reunion} = $donnees->{query}->param('date_reunion');
#  $donnees->{ec} = $donnees->{query}->param('ec');
#  warn "***LOGIN: ", $donnees->{login}, "\n";
  $donnees->{warn_login} = "";
  verif_login($donnees);

  print $donnees->{query}->header(-type => "text/html",
				  -charset => 'ISO-8859-1');

  if (not defined $donnees->{login}
      or (not defined $permanents{$donnees->{login}}
	  and not defined $login_valides{$donnees->{login}})) {
    login($donnees);
  } else {
    $donnees->{nom} = $permanents{$donnees->{login}} ||
      $login_valides{$donnees->{login}};
    $donnees->{action} = $donnees->{query}->param('action');
    if (not defined $donnees->{action}
	or not defined $action{$donnees->{action}}) {
      if (defined $donnees->{action}) {
	# l'action demandee est inconnue
	# pour l'instant, on sort...
	print $donnees->{query}->p("Quelle mthode ?");
      }
      # l'action par defaut
      $donnees->{action} = 'menu';
    }
    $action{$donnees->{action}}($donnees);
  }
}

#------------------------------------------------------------
# routines de vrification et affichage du login.
#------------------------------------------------------------

sub verif_login {
  my $donnees = shift;

  if (not defined $donnees->{login}) {
    $donnees->{query}->param('login', undef);
  } elsif (not defined $permanents{$donnees->{login}} and
	   not defined $login_valides{$donnees->{login}}) {
    $donnees->{warn_login} = "Mauvais Login, recommencez !";
    $donnees->{query}->param('login', undef);
    return 1;
  }

  if ($donnees->{login} eq $chef) {
    $donnees->{login_chef} = 1;
  } else {
    $donnees->{login_chef} = 0;
  }
}

sub login {
  my $donnees = shift;
  my $template = Template->new();
  my $fichier = "login.html";

  $template->process($fichier, $donnees)
    || die "Template process failed in login(): ",
      $template->error(), "\n";
}

#------------------------------------------------------------
# routine de choix d'action
#------------------------------------------------------------

sub choix_action {
  my $donnees = shift;
  $donnees->{nom} = $permanents{$donnees->{login}} ||
    $login_valides{$donnees->{login}};
  warn "ACTION(donnees): ", $donnees->{action}, "\n";
  warn "ACTION(query)", $donnees->{query}->param('action'), "\n";
  $donnees->{action} = undef;

  if ($donnees->{login} eq $chef) {
    choix_action_chef($donnees);
  } elsif (defined $permanents{$donnees->{login}}) {
    choix_action_permanent($donnees);
  } elsif (defined $login_valides{$donnees->{login}}) {
    choix_action_autre($donnees);
  }
}

sub choix_action_chef {
  my $donnees = shift;
  my $template = Template->new();
  my $fichier = "choix_action_chef.html";

  $template->process($fichier, $donnees)
    || die "Template process failed in choix_action(): ",
      $template->error(), "\n";

  $donnees->{login} = $donnees->{query}->param('login');
  $donnees->{action} = $donnees->{query}->param('action');

}

sub choix_action_permanent {
  my $donnees = shift;
  my $template = Template->new();
  my $fichier = "choix_action_permanent.html";

  $template->process($fichier, $donnees)
    || die "Template process failed in choix_action(): ",
      $template->error(), "\n";

  $donnees->{login} = $donnees->{query}->param('login');
  $donnees->{action} = $donnees->{query}->param('action');

}

sub choix_action_autre {
  my $donnees = shift;

  # les autres ne peuvent que consulter
  $donnees->{action} = 'consultation';
  $action{$donnees->{action}}($donnees);
}

#-----------------------------------------------------------
# routines de saisie (et vrification) et modification
#------------------------------------------------------------

sub saisie {
  my $donnees = shift;
  my $template = Template->new();
  my $fichier = "saisie.html";
  $donnees->{login} 	  = $donnees->{query}->param('login');
  $donnees->{nom} 	  = $permanents{$donnees->{login}};
  $donnees->{action} 	  = $donnees->{query}->param('action');
  $donnees->{listejour}   = $ListeJour;
  $donnees->{listeannee}  = $ListeAnnee;

  #afficher le formulaire de saisie (grer l'ventuel warning)
  $template->process($fichier, $donnees)
    || die "Template process failed in saisie(): ",
      $template->error(), "\n";
}

sub saisie_chef {
  my $donnees = shift;
  my $template = Template->new();
  $donnees->{login} 	  = $donnees->{query}->param('login');
  $donnees->{nom} 	  = $permanents{$donnees->{login}};
  $donnees->{action} 	  = $donnees->{query}->param('action');
  $donnees->{listejour}   = $ListeJour;
  $donnees->{listeannee}  = $ListeAnnee;
  my $fichier = "saisie_chef.html";

  #afficher le formulaire de saisie (grer l'ventuel warning)
  $template->process($fichier, $donnees)
    || die "Template process failed in saisie_chef(): ",
      $template->error(), "\n";
}

sub verif_saisie {
  my $donnees = shift;
  my $warning = undef;

  {
    my $q = $donnees->{query};
    foreach my $field_name (qw{
			       mois_debut annee_debut mois_fin
			       annee_fin jour_reunion mois_reunion
			       annee_reunion texte_libre theses
			       projets congres1_nom congres1_theme
			       congres1_mois congres1_annee
			       congres1_lieu congres2_nom
			       congres2_theme congres2_mois
			       congres2_annee congres2_lieu
			       congres3_nom congres3_theme
			       congres3_mois congres3_annee
			       congres3_lieu congres_texte publi1_nom
			       publi1_auteurs publi1_type publi1_etat
			       publi1_theme publi2_nom publi2_auteurs
			       publi2_type publi2_etat publi2_theme
			       publi3_nom publi3_auteurs publi3_type
			       publi3_etat publi3_theme publi4_nom
			       publi4_auteurs publi4_type publi4_etat
			       publi4_theme publi5_nom publi5_auteurs
			       publi5_type publi5_etat publi5_theme
			       publi6_nom publi6_auteurs publi6_type
			       publi6_etat publi6_theme publi7_nom
			       publi7_auteurs publi7_type publi7_etat
			       publi7_theme publi_texte scientifique
			       enseignement vie divers
			     }) {
      $donnees->{$field_name} = $q->param($field_name) || "";
    }
  }


  $donnees->{warn_saisie}   = "";


  {
    my ($annee, $mois, $jour) = (
				 $donnees->{annee_reunion},
				 $donnees->{mois_reunion},
			 	 $donnees->{jour_reunion},
				);
    if (not defined $annee or $annee eq "00" or
	not defined $mois or $mois eq "00" or
	not defined $jour or $jour eq "00") {
      $warning .=
	"Vous devez dfinir une date prcise de runion (jj/mm/aaaa). ";
    }
  }

  {
    my ($annee, $mois) = (
			  $donnees->{annee_debut},
			  $donnees->{mois_debut},
			 );
    if (not defined $annee or $annee eq "00" or
	not defined $mois or $mois eq "00") {
      $warning .=
	"Vous devez dfinir un dbut de priode (mm/aaaa). ";
    }
  }

  {
    my ($annee, $mois) = (
			  $donnees->{annee_fin},
			  $donnees->{mois_fin},
			 );
    if (not defined $annee or $annee eq "00" or
	not defined $mois or $mois eq "00") {
      $warning .=
	"Vous devez dfinir une fin de priode (mm/aaaa). ";
    }
  }

  if (defined $warning) {
    $donnees->{warn_saisie} = $warning;
    $donnees->{action} = "saisie_chef"
      if ($donnees->{login} eq $chef);
    $donnees->{action} = "saisie"
      if ($donnees->{login} ne $chef and
	  defined $permanents{$donnees->{login}});
  } elsif (defined $permanents{$donnees->{login}}) {
    $donnees->{action} = "generation_chef"
      if ($donnees->{login} eq $chef);
    $donnees->{action} = "generation"
      if ($donnees->{login} ne $chef);
  }
  $action{$donnees->{action}}($donnees);
}


#------------------------------------------------------------
# routines de modification des rapports
#------------------------------------------------------------

sub modif {
  my $donnees = shift;
  my $template = Template->new();
  my @files;
  my @periode;
  my $file;

  for (parse_dir(`ls -lt $repository`)) {
    my ($name, $type, $size, $mtime, $mode) = @$_;
    next if (not $name =~ m/$donnees->{login}/);
    push @files, $name;
  }

### en classant par 'ls -lt', le premier fichier est le bon.
### (ce classement est donc inutile - et faux).
#   foreach my $d (@files) {
#     $d =~ s/[a-z]*-([0-9]*)\.xml/$1/;
#     my $fd = $d;
#     $fd =~ s/([0-9][0-9]?)([0-9][0-9])([0-9][0-9][0-9][0-9])/$3$2$1/;
#     push @periode, $fd;
#   }
#   my @date = sort {$b <=> $a} @periode;
#   my $lastdate = $date[0];
#   $lastdate =~ s/([0-9][0-9][0-9][0-9])([0-9][0-9])([0-9][0-9]?)/$3$2$1/;
#  $file = $donnees->{login} . "-" . $lastdate . ".xml";

  $file = @files[0];

  $donnees->{rapport} = $file;
  parse($donnees);

  warn "$donnees->{mois_debut}, $donnees->{mois_fin}";

  if ($donnees->{login} =~ m/$chef/) {
    my $fichier = "saisie_chef.html";
    $donnees->{login} 	  = $donnees->{query}->param('login');
    $donnees->{nom} 	  = $permanents{$donnees->{login}};
    $donnees->{action} 	  = $donnees->{query}->param('action');
    $donnees->{listejour}   = $ListeJour;
    $donnees->{listeannee}  = $ListeAnnee;

    #afficher le formulaire de saisie (grer l'ventuel warning)
    $template->process($fichier, $donnees)
      || die "Template process failed in saisie(): ",
	$template->error(), "\n";
  } else {
    my $fichier = "saisie.html";
    $donnees->{login} 	  = $donnees->{query}->param('login');
    $donnees->{nom} 	  = $permanents{$donnees->{login}};
    $donnees->{action} 	  = $donnees->{query}->param('action');
    $donnees->{listejour}   = $ListeJour;
    $donnees->{listeannee}  = $ListeAnnee;

    #afficher le formulaire de saisie (grer l'ventuel warning)
    $template->process($fichier, $donnees)
      || die "Template process failed in saisie(): ",
	$template->error(), "\n";
  }
}

#------------------------------------------------------------
# routines de cration des rapports
#------------------------------------------------------------

sub creer_rapport {
  my $donnees = shift;
  my $rapport;
  my $line;
  my $flash;
  $donnees->{nom} = $permanents{$donnees->{login}};

  # gnrer un nom de fichier
  $rapport = gen_fichier($donnees);

  #remplacer les variables du Template par les donnees de $donnees
  open my $src, "<", "Template.xml";
  while (defined ($line = <$src>)) {
    $line =~
      s{\[% ([a-z0-9_]+) %\]}{
	my $value = "";
	if (not defined $donnees->{$1}) {
	  $value = "BUG($1)";
	} else {
	  $value = $donnees->{$1};
	  $value =~ s/&/&amp;/g;
	  $value =~ s/</&lt;/g;
	  $value =~ s/>/&gt;/g;
	}
	$value;
      }ge;

    $flash .= $line;
  }
  close $src;
  my ($RAP, $file) = tempfile('flashXXXX',
			      SUFFIX => '.xml',
			      DIR    => $repository);
  print $RAP $flash;
  close($RAP);
  # copier dans $rapport, puis modifier les droits
  my $destination = $repository . $rapport;
  move("$file", "$destination");
  chmod 0664, "$destination";

  # retour au choix d'action
  $donnees->{action} = 'retour';
  $action{$donnees->{action}}($donnees);
}

sub creer_rapport_chef {
  my $donnees = shift;
  my $rapport;
  my $line;
  my $flash;
  $donnees->{nom} = $permanents{$donnees->{login}};

  # gnrer un nom de fichier
  $rapport = gen_fichier($donnees);

  #remplacer les variables du Template par les donnees de $donnees
  open my $src, "<", "Template_chef.xml";
  while (defined ($line = <$src>)) {
    $line =~
      s{\[% ([a-z0-9_]+) %\]}{
	my $value = "";
	if (not defined $donnees->{$1}) {
	  $value = "BUG($1)";
	} else {
	  $value = $donnees->{$1};
	  $value =~ s/&/&amp;/g;
	  $value =~ s/</&lt;/g;
	  $value =~ s/>/&gt;/g;
	}
	$value;
      }ge;

    $flash .= $line;
  }
  close $src;
  my ($RAP, $file) = tempfile('flashXXXX',
			      SUFFIX => '.xml',
			      DIR    => $repository);
  print $RAP $flash;
  close($RAP);
  # copier dans $rapport, puis modifier les droits
  my $destination = $repository . $rapport;
  move("$file", "$destination");
  chmod 0664, "$destination";

  # retour au choix d'action
  $donnees->{action} = 'retour';
  $action{$donnees->{action}}($donnees);
}

sub gen_fichier {
  my $donnees = shift;
  my ($jour, $mois, $annee) = (
			       $donnees->{jour_reunion},
			       $donnees->{mois_reunion},
			       $donnees->{annee_reunion},
			      );
  my $fichier = $donnees->{login};

  $fichier =~ m/^([a-z]*)$/;
  $fichier = $1;
  $jour =~ m/^([0-9]*)$/;
  $fichier .= "-" . $1;
  $mois =~ m/^([0-9]*)$/;
  $fichier .= $1;
  $annee =~ m/^([0-9]*)$/;
  $fichier .= $1;

  #gnrer un nom de fichier
  #nom-datereunion.xml -> dupont-26102005.xml
  $fichier .= ".xml";

  return $fichier;
}

#------------------------------------------------------------
# routines de consultation des rapports
#------------------------------------------------------------

sub consult {
  my $donnees = shift;
  my @date;
  my @ec;
  my @filee;
  my @filep;
  my $template = Template->new();
  my $fichier = "choix_periode_ec.html";

  # rcupration des fichiers XML dans le repository
  for (parse_dir(`ls -l $repository`)) {
    my ($name, $type, $size, $mtime, $mode) = @$_;
    next if (not $name =~ m/([a-z]*)-[0-9]*\.xml/);
    push @filee, $name;
  }
  @filep = @filee;

  # rcupration des login disponibles
  foreach my $perm (@filee) {
    $perm =~ s/([a-z]*)-[0-9]*\.xml/$1/;
    if (not join(' ', @ec) =~ m/$permanents{$perm}/) {
      push @ec, $permanents{$perm};
    }
#    next if (exists $ec[$permanents{$perm}]);
  }
  $donnees->{liste_ec} = \@ec;

  # rcupration des dates de runion disponibles
  foreach my $d (@filep) {
    $d =~ s/[a-z]*-([0-9]*)\.xml/$1/;
    my $fd = $d;
    $fd =~ s/([0-9][0-9]?)([0-9][0-9])([0-9][0-9][0-9][0-9])/$1\/$2\/$3/;
    warn "@date";
    if (not join(' ', @date) =~ m/$fd/) {
      push @date, $fd;
    }
#    next if (exists $date{$fd});
  }
  $donnees->{date_reunion} = \@date;

  #afficher le formulaire de choix des dates et/ou ec
  $template->process($fichier, $donnees)
    || die "Template process failed in choix_periode_ec(): ",
      $template->error(), "\n";
}

sub select_rap {
  my $donnees = shift;
  my $login = "*";
  my @files;
  # rcupration des donnes en param
  $donnees->{date_reunion} = $donnees->{query}->param('date_reunion');
  $donnees->{ec}           = $donnees->{query}->param('ec');

  # rcupration de la date et de l'ec au bon format (ou "*")
  if (defined $donnees->{date_reunion}
      and $donnees->{date_reunion} ne "00") {
    $donnees->{date_reunion} =~ s/\///g;
  } else {
    $donnees->{date_reunion} = "*";
  }

  foreach my $log (keys %permanents) {
    if ($permanents{$log} eq $donnees->{ec}) {
      $login = $log;
      last;
    }
  }
  $donnees->{ec} = $login;
  my $fichier = $donnees->{ec}."-".$donnees->{date_reunion}.".xml";

  # rcuprer les fichiers correspondants  cette date et/ou ec
  $fichier =~ s/\*/\.\*/g;
  for (parse_dir(`ls -l $repository`)) {
    my ($name, $type, $size, $mtime, $mode) = @$_;
    next if (not $name =~ m/$fichier/);
    push @files, $name;
  }

  $donnees->{fichiers} = \@files;
  # appeler l'affichage de la liste de ces fichiers avec des liens
  aff_list_files($donnees);
}

sub aff_list_files {
  my $donnees = shift;
  my $template = Template->new();
  my $fichier = "choix_rapports.html";

  #afficher le formulaire de choix des dates et/ou ec
  $template->process($fichier, $donnees)
    || die "Template process failed in aff_list_files(): ",
      $template->error(), "\n";
}

sub list_rap_to_process {
  my $donnees = shift;
  my $file;

  # rcupration des paramtres
  foreach my $field (keys %{$donnees->{query}}) {
    $donnees->{$field} = $donnees->{query}->param('$field');
  }

  $file = $donnees->{query}->param('fichier');

#   warn "$file";
#   $file =~ s/^(.*\.xml)$/$1/;

  # ajout dans $donnees puis appel du parser
  $donnees->{rapport} = $file;
  parse($donnees);
  display($donnees);
}

sub parse {
  my $donnees = shift;
  my $twig= new XML::Twig;
  my $file = $donnees->{rapport};

  # rcupration des donnes communes
  my $fichier = $repository . $file;
  $twig->parsefile("$fichier");
  my $root = $twig->root;
  my @elements = $root->children();
  my @titre = $elements[0]->children;
  my $ec = $titre[0]->first_child_text;
  my @periode = $titre[1]->children;
  $donnees->{mois_debut} = $periode[0]->first_child_text('Mois');
  $donnees->{annee_debut} = $periode[0]->first_child_text('Annee');
  $donnees->{date_debut} =
    $ListeMois->{$donnees->{mois_debut}} . " " . $donnees->{annee_debut};
  $donnees->{mois_fin} = $periode[1]->first_child_text('Mois');
  $donnees->{annee_fin} = $periode[1]->first_child_text('Annee');
  $donnees->{date_fin} =
    $ListeMois->{$donnees->{mois_fin}} . " " . $donnees->{annee_fin};

  $donnees->{jour_reunion} = $titre[2]->last_child->first_child_text('Jour');
  $donnees->{mois_reunion} = $titre[2]->last_child->first_child_text('Mois');
  $donnees->{annee_reunion} = $titre[2]->last_child->first_child_text('Annee');

  $donnees->{date_reunion} =
    $donnees->{jour_reunion} . " " .
      $ListeMois->{$donnees->{mois_reunion}}
	. " " . $donnees->{annee_reunion};
  $donnees->{ec} = $ec;

  if ($donnees->{ec} =~ m/Dupont/) {
    $donnees->{texte_libre} =	$elements[1]->last_child_text;
  } else {
    $donnees->{theses}         = $elements[1]->first_child_text;
    $donnees->{projets}        = $elements[2]->first_child_text;
    $donnees->{congres1_nom}   = $elements[3]->first_child_text;
    $donnees->{congres1_theme} =
      $elements[3]->first_child_text('Theme');
    $donnees->{congres1_mois}  =
      $ListeMois->{$elements[3]->last_child->first_child_text('Mois')};
    $donnees->{congres1_annee} =
      $elements[3]->last_child->first_child_text('Annee');
    $donnees->{congres1_annee} = ""
      if ($donnees->{congres1_annee} eq "00");
    $donnees->{congres1_lieu}  =
      $elements[3]->first_child_text('Lieu');
    $donnees->{congres2_nom}   = $elements[4]->first_child_text;
    $donnees->{congres2_theme} =
      $elements[4]->first_child_text('Theme');
    $donnees->{congres2_mois}  =
      $ListeMois->{$elements[4]->last_child->first_child_text('Mois')};
    $donnees->{congres2_annee} =
      $elements[4]->last_child->first_child_text('Annee');
    $donnees->{congres2_annee} = ""
      if ($donnees->{congres2_annee} eq "00");
    $donnees->{congres2_lieu}  =
      $elements[4]->first_child_text('Lieu');
    $donnees->{congres3_nom}   = $elements[5]->first_child_text;
    $donnees->{congres3_theme} =
      $elements[5]->first_child_text('Theme');
    $donnees->{congres3_mois}  =
      $ListeMois->{$elements[5]->last_child->first_child_text('Mois')};
    $donnees->{congres3_annee} =
      $elements[5]->last_child->first_child_text('Annee');
    $donnees->{congres3_annee} = ""
      if ($donnees->{congres3_annee} eq "00");
    $donnees->{congres3_lieu}  =
      $elements[5]->first_child_text('Lieu');
    $donnees->{congres_texte}  = $elements[6]->first_child_text;
    $donnees->{publi1_nom}     = $elements[7]->first_child_text;
    $donnees->{publi1_auteurs}  =
      $elements[7]->first_child_text('Auteurs');
    $donnees->{publi1_type}    =
      $elements[7]->first_child_text('Type');
    $donnees->{publi1_type} = ""
      if ($donnees->{publi1_type} eq "00");
    $donnees->{publi1_etat}    =
      $elements[7]->first_child_text('Etat');
    $donnees->{publi1_etat} = ""
      if ($donnees->{publi1_etat} eq "00");
    $donnees->{publi1_theme}   =
      $elements[7]->first_child_text('Theme');
    $donnees->{publi2_nom}     = $elements[8]->first_child_text;
    $donnees->{publi2_auteurs}  =
      $elements[8]->first_child_text('Auteurs');
    $donnees->{publi2_type}    =
      $elements[8]->first_child_text('Type');
    $donnees->{publi2_type} = ""
      if ($donnees->{publi2_type} eq "00");
    $donnees->{publi2_etat}    =
      $elements[8]->first_child_text('Etat');
    $donnees->{publi2_etat} = ""
      if ($donnees->{publi2_etat} eq "00");
    $donnees->{publi2_theme}   =
      $elements[8]->first_child_text('Theme');
    $donnees->{publi3_nom}     = $elements[9]->first_child_text;
    $donnees->{publi3_auteurs}  =
      $elements[9]->first_child_text('Auteurs');
    $donnees->{publi3_type}    =
      $elements[9]->first_child_text('Type');
    $donnees->{publi3_type} = ""
      if ($donnees->{publi3_type} eq "00");
    $donnees->{publi3_etat}    =
      $elements[9]->first_child_text('Etat');
    $donnees->{publi3_etat} = ""
      if ($donnees->{publi3_etat} eq "00");
    $donnees->{publi3_theme}   =
      $elements[9]->first_child_text('Theme');
    $donnees->{publi4_nom}     = $elements[10]->first_child_text;
    $donnees->{publi4_auteurs}  =
      $elements[10]->first_child_text('Auteurs');
    $donnees->{publi4_type}    =
      $elements[10]->first_child_text('Type');
    $donnees->{publi4_type} = ""
      if ($donnees->{publi4_type} eq "00");
    $donnees->{publi4_etat}    =
      $elements[10]->first_child_text('Etat');
    $donnees->{publi4_etat} = ""
      if ($donnees->{publi4_etat} eq "00");
    $donnees->{publi4_theme}   =
      $elements[10]->first_child_text('Theme');
    $donnees->{publi5_nom}     = $elements[11]->first_child_text;
    $donnees->{publi5_auteurs}  =
      $elements[11]->first_child_text('Auteurs');
    $donnees->{publi5_type}    =
      $elements[11]->first_child_text('Type');
    $donnees->{publi5_type} = ""
      if ($donnees->{publi5_type} eq "00");
    $donnees->{publi5_etat}    =
      $elements[11]->first_child_text('Etat');
    $donnees->{publi5_etat} = ""
      if ($donnees->{publi5_etat} eq "00");
    $donnees->{publi5_theme}   =
      $elements[11]->first_child_text('Theme');
    $donnees->{publi6_nom}     = $elements[12]->first_child_text;
    $donnees->{publi6_auteurs}  =
      $elements[12]->first_child_text('Auteurs');
    $donnees->{publi6_type}    =
      $elements[12]->first_child_text('Type');
    $donnees->{publi6_type} = ""
      if ($donnees->{publi6_type} eq "00");
    $donnees->{publi6_etat}    =
      $elements[12]->first_child_text('Etat');
    $donnees->{publi6_etat} = ""
      if ($donnees->{publi6_etat} eq "00");
    $donnees->{publi6_theme}   =
      $elements[12]->first_child_text('Theme');
    $donnees->{publi7_nom}     = $elements[13]->first_child_text;
    $donnees->{publi7_auteurs}  =
      $elements[13]->first_child_text('Auteurs');
    $donnees->{publi7_type}    =
      $elements[13]->first_child_text('Type');
    $donnees->{publi7_type} = ""
      if ($donnees->{publi7_type} eq "00");
    $donnees->{publi7_etat}    =
      $elements[13]->first_child_text('Etat');
    $donnees->{publi7_etat} = ""
      if ($donnees->{publi7_etat} eq "00");
    $donnees->{publi7_theme}   =
      $elements[13]->first_child_text('Theme');
    $donnees->{publi_texte}    = $elements[14]->first_child_text;
    $donnees->{scientifique}   = $elements[15]->first_child_text;
    $donnees->{enseignement}   = $elements[16]->first_child_text;
    $donnees->{vie}   	       = $elements[17]->first_child_text;
    $donnees->{divers}         = $elements[18]->first_child_text;
  }

  warn "$donnees->{congres_texte}\n";
}

sub display {
  my $donnees = shift;
  my $template = Template->new();
  my $html;

  if ($donnees->{ec} =~ m/Dupont/) {
    $html = "rapport_chef.html";
  } else {
    $html = "rapport.html";
  }
  $template->process($html, $donnees)
    || die "Template process failed in display(): ",
      $template->error(), "\n";
}

__END__
